@@ -17,14 +17,14 @@
 
 // Keep these in sync with other top-level files.
 #![allow(clippy::await_holding_refcell_ref)]
+#![allow(clippy::collapsible_else_if)]
 #![warn(anonymous_parameters, bad_style, missing_docs)]
 #![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
 #![warn(unsafe_code)]
 
 use endbasic_core::exec::{Machine, StopReason};
 use endbasic_std::console::{self, Console};
-use endbasic_std::store::Store;
-use futures_lite::future::block_on;
+use endbasic_std::storage::Storage;
 use std::cell::RefCell;
 use std::io;
 use std::rc::Rc;
@@ -38,34 +38,36 @@ pub fn print_welcome(console: Rc<RefCell<dyn Console>>) -> io::Result<()> {
     console.print(&format!("    Welcome to EndBASIC {}.", env!("CARGO_PKG_VERSION")))?;
     console.print("")?;
     console.print("    Type HELP for interactive usage information.")?;
-    console.print("    Type LOAD \"DEMO:TOUR.BAS\": RUN for a guided tour.")?;
+    console.print("    For a guided tour, type: LOAD \"DEMOS:/TOUR.BAS\": RUN")?;
     console.print("")?;
     Ok(())
 }
 
-/// Loads the `AUTOEXEC.BAS` file if it exists in the `store`.
+/// Loads the `AUTOEXEC.BAS` file if it exists in the `drive`.
 ///
 /// Failures to process the file are logged to the `console` but are ignored.  Other failures are
 /// returned.
-pub fn try_load_autoexec(
+pub async fn try_load_autoexec(
     machine: &mut Machine,
     console: Rc<RefCell<dyn Console>>,
-    store: Rc<RefCell<dyn Store>>,
+    storage: Rc<RefCell<Storage>>,
 ) -> io::Result<()> {
-    match store.borrow().get("AUTOEXEC.BAS") {
-        Ok(code) => {
-            console.borrow_mut().print("Loading AUTOEXEC.BAS...")?;
-            match block_on(machine.exec(&mut code.as_bytes())) {
-                Ok(_) => Ok(()),
-                Err(e) => {
-                    console.borrow_mut().print(&format!("AUTOEXEC.BAS failed: {}", e))?;
-                    Ok(())
-                }
-            }
-        }
-        Err(e) if e.kind() == io::ErrorKind::NotFound => Ok(()),
+    let code = match storage.borrow().get("AUTOEXEC.BAS").await {
+        Ok(code) => code,
+        Err(e) if e.kind() == io::ErrorKind::NotFound => return Ok(()),
         Err(e) => {
-            console.borrow_mut().print(&format!("AUTOEXEC.BAS exists but cannot be read: {}", e))
+            return console
+                .borrow_mut()
+                .print(&format!("AUTOEXEC.BAS exists but cannot be read: {}", e));
+        }
+    };
+
+    console.borrow_mut().print("Loading AUTOEXEC.BAS...")?;
+    match machine.exec(&mut code.as_bytes()).await {
+        Ok(_) => Ok(()),
+        Err(e) => {
+            console.borrow_mut().print(&format!("AUTOEXEC.BAS failed: {}", e))?;
+            Ok(())
         }
     }
 }
@@ -79,13 +81,14 @@ pub async fn run_repl_loop(
     console: Rc<RefCell<dyn Console>>,
 ) -> io::Result<i32> {
     let mut stop_reason = StopReason::Eof;
+    let mut history = vec![];
     while stop_reason == StopReason::Eof {
         let line = {
             let mut console = console.borrow_mut();
             if console.is_interactive() {
                 console.print("Ready")?;
             }
-            console::read_line(&mut *console, "", "").await
+            console::read_line(&mut *console, "", "", Some(&mut history)).await
         };
 
         match line {
@@ -120,18 +123,22 @@ pub async fn run_repl_loop(
 mod tests {
     use super::*;
     use endbasic_std::testutils::Tester;
+    use futures_lite::future::block_on;
 
     #[test]
     fn test_autoexec_ok() {
-        let autoexec = "PRINT \"hello\": global_var = 3";
+        // The code in the autoexec test file should access, in a mutable fashion, all the resources
+        // that the try_load_autoexec function uses to ensure the function's code doesn't hold onto
+        // references while executing the autoexec code and causing a borrowing violation.
+        let autoexec = "PRINT \"hello\": global_var = 3: CD \"MEMORY:/\"";
         let mut tester = Tester::default().write_file("AUTOEXEC.BAS", autoexec);
-        let (console, store) = (tester.get_console(), tester.get_store());
-        try_load_autoexec(tester.get_machine(), console, store).unwrap();
+        let (console, storage) = (tester.get_console(), tester.get_storage());
+        block_on(try_load_autoexec(tester.get_machine(), console, storage)).unwrap();
         tester
             .run("")
             .expect_var("global_var", 3)
             .expect_prints(["Loading AUTOEXEC.BAS...", "hello"])
-            .expect_file("AUTOEXEC.BAS", autoexec)
+            .expect_file("MEMORY:/AUTOEXEC.BAS", autoexec)
             .check();
     }
 
@@ -139,8 +146,8 @@ mod tests {
     fn test_autoexec_error_is_ignored() {
         let autoexec = "a = 1: b = undef: c = 2";
         let mut tester = Tester::default().write_file("AUTOEXEC.BAS", autoexec);
-        let (console, store) = (tester.get_console(), tester.get_store());
-        try_load_autoexec(tester.get_machine(), console, store).unwrap();
+        let (console, storage) = (tester.get_console(), tester.get_storage());
+        block_on(try_load_autoexec(tester.get_machine(), console, storage)).unwrap();
         tester
             .run("after = 5")
             .expect_var("a", 1)
@@ -149,7 +156,7 @@ mod tests {
                 "Loading AUTOEXEC.BAS...",
                 "AUTOEXEC.BAS failed: Undefined variable undef",
             ])
-            .expect_file("AUTOEXEC.BAS", autoexec)
+            .expect_file("MEMORY:/AUTOEXEC.BAS", autoexec)
             .check();
     }
 
@@ -158,22 +165,22 @@ mod tests {
         let mut tester = Tester::default()
             .write_file("AUTOEXEC.BAS", "a = 1")
             .write_file("autoexec.bas", "a = 2");
-        let (console, store) = (tester.get_console(), tester.get_store());
-        try_load_autoexec(tester.get_machine(), console, store).unwrap();
+        let (console, storage) = (tester.get_console(), tester.get_storage());
+        block_on(try_load_autoexec(tester.get_machine(), console, storage)).unwrap();
         tester
             .run("")
             .expect_var("a", 1)
             .expect_prints(["Loading AUTOEXEC.BAS..."])
-            .expect_file("AUTOEXEC.BAS", "a = 1")
-            .expect_file("autoexec.bas", "a = 2")
+            .expect_file("MEMORY:/AUTOEXEC.BAS", "a = 1")
+            .expect_file("MEMORY:/autoexec.bas", "a = 2")
             .check();
     }
 
     #[test]
     fn test_autoexec_missing() {
         let mut tester = Tester::default();
-        let (console, store) = (tester.get_console(), tester.get_store());
-        try_load_autoexec(tester.get_machine(), console, store).unwrap();
+        let (console, storage) = (tester.get_console(), tester.get_storage());
+        block_on(try_load_autoexec(tester.get_machine(), console, storage)).unwrap();
         tester.run("").check();
     }
 }
