@@ -17,19 +17,20 @@
 
 // Keep these in sync with other top-level files.
 #![allow(clippy::await_holding_refcell_ref)]
+#![allow(clippy::collapsible_else_if)]
 #![warn(anonymous_parameters, bad_style, missing_docs)]
 #![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
 #![warn(unsafe_code)]
 
 use anyhow::{anyhow, Result};
-use endbasic_std::store::{FileStore, Store};
+use endbasic_std::storage::Storage;
 use endbasic_std::terminal::TerminalConsole;
-use futures_lite::future::block_on;
 use getopts::Options;
 use std::cell::RefCell;
 use std::env;
 use std::fs::File;
-use std::path::{Path, PathBuf};
+use std::io;
+use std::path::Path;
 use std::process;
 use std::rc::Rc;
 
@@ -68,8 +69,8 @@ fn program_name(mut args: env::Args, default_name: &'static str) -> (String, env
 fn help(name: &str, opts: &Options) {
     let brief = format!("Usage: {} [options] [program-file]", name);
     println!("{}", opts.usage(&brief));
-    println!("Report bugs to: https://github.com/jmmv/endbasic/issues");
-    println!("EndBASIC home page: https://github.com/jmmv/endbasic");
+    println!("Report bugs to: https://github.com/endbasic/endbasic/issues");
+    println!("EndBASIC home page: https://www.endbasic.dev/");
 }
 
 /// Prints version information following the GNU Standards format.
@@ -79,83 +80,95 @@ fn version() {
     println!("License Apache Version 2.0 <http://www.apache.org/licenses/LICENSE-2.0>");
 }
 
-/// Computes the path to the directory where user programs live if `flag` is none; otherwise
-/// just returns `flag`.
-fn get_programs_dir(flag: Option<String>) -> Result<PathBuf> {
-    let dir = flag.map(PathBuf::from).or_else(|| {
-        dirs::document_dir().map(|d| d.join("endbasic")).or_else(|| {
-            // On Linux, dirs::document_dir() seems to return None whenever user-dirs.dirs is
-            // not present... which is suboptimal.  Compute a reasonable default based on the
-            // home directory.
-            dirs::home_dir().map(|h| h.join("Documents/endbasic"))
-        })
+/// Returns `flag` if present, or else returns the URI of the default `LOCAL` drive.
+fn get_local_drive_spec(flag: Option<String>) -> Result<String> {
+    let dir = flag.or_else(|| {
+        dirs::document_dir().map(|d| format!("file://{}", d.join("endbasic").display())).or_else(
+            || {
+                // On Linux, dirs::document_dir() seems to return None whenever user-dirs.dirs is
+                // not present... which is suboptimal.  Compute a reasonable default based on the
+                // home directory.
+                dirs::home_dir()
+                    .map(|h| format!("file://{}", h.join("Documents/endbasic").display()))
+            },
+        )
     });
 
     // Instead of aborting on a missing programs directory, we could disable the LOAD/SAVE commands
     // when we cannot compute this folder, but that seems like hiding a corner case that is unlikely
     // to surface.  A good reason to do this, however, would be to allow the user to explicitly
     // disable this functionality to keep the interpreter from touching the disk.
-    if dir.is_none() {
-        return Err(anyhow!("Cannot compute default path to the Documents folder"));
+    match dir {
+        Some(dir) => Ok(dir),
+        None => Err(anyhow!("Cannot compute default path to the Documents folder")),
     }
-    Ok(dir.unwrap())
 }
 
-/// Creates a new store backed by `dir` and overlays the built-in demos.
-fn new_store_with_demos(dir: &Path) -> Rc<RefCell<dyn Store>> {
-    if dir == Path::new(":memory:") {
-        Rc::from(RefCell::from(endbasic::demos::DemoStoreOverlay::new(
-            endbasic_std::store::InMemoryStore::default(),
-        )))
-    } else {
-        Rc::from(RefCell::from(endbasic::demos::DemoStoreOverlay::new(FileStore::new(dir))))
-    }
+/// Sets up the common storage drives.
+///
+/// This instantiates non-optional drives, such as `MEMORY:` and `DEMOS:`, maps `LOCAL` the
+/// location given in `local_drive_spec`.
+pub fn setup_storage(storage: &mut Storage, local_drive_spec: &str) -> io::Result<()> {
+    storage.register_scheme("demos", Box::from(endbasic::demos::DemoDriveFactory::default()));
+    storage.mount("demos", "demos://").expect("Demos drive shouldn't fail to mount");
+    storage.register_scheme(
+        "file",
+        Box::from(endbasic_std::storage::DirectoryDriveFactory::default()),
+    );
+    storage.mount("local", local_drive_spec)?;
+    storage.cd("local:").expect("Local drive was just registered");
+    Ok(())
 }
 
 /// Enters the interactive interpreter.
 ///
-/// `dir` specifies the directory that the interpreter will use for any commands that manipulate
-/// files.  The special name `:memory:` makes the interpreter use an in-memory only store.
-fn run_repl_loop(dir: &Path) -> endbasic_core::exec::Result<i32> {
+/// `local_drive` is the optional local drive to mount and use as the default location.
+async fn run_repl_loop(local_drive_spec: &str) -> endbasic_core::exec::Result<i32> {
     let console = Rc::from(RefCell::from(TerminalConsole::from_stdio()?));
-    let store = new_store_with_demos(dir);
-    let mut machine = endbasic_std::MachineBuilder::default()
-        .with_console(console.clone())
-        .make_interactive()
-        .with_store(store.clone())
-        .build()?;
+    let mut builder =
+        endbasic_std::MachineBuilder::default().with_console(console.clone()).make_interactive();
+
+    let storage = builder.get_storage();
+    setup_storage(&mut storage.borrow_mut(), local_drive_spec)?;
+
+    let mut machine = builder.build()?;
     endbasic::print_welcome(console.clone())?;
-    endbasic::try_load_autoexec(&mut machine, console.clone(), store)?;
-    Ok(block_on(endbasic::run_repl_loop(&mut machine, console))?)
+    endbasic::try_load_autoexec(&mut machine, console.clone(), storage).await?;
+    Ok(endbasic::run_repl_loop(&mut machine, console).await?)
 }
 
 /// Executes the `path` program in a fresh machine.
-fn run_script<P: AsRef<Path>>(path: P) -> endbasic_core::exec::Result<i32> {
+async fn run_script<P: AsRef<Path>>(path: P) -> endbasic_core::exec::Result<i32> {
     let mut machine = endbasic_std::MachineBuilder::default().build()?;
     let mut input = File::open(path)?;
-    Ok(block_on(machine.exec(&mut input))?.as_exit_code())
+    Ok(machine.exec(&mut input).await?.as_exit_code())
 }
 
 /// Executes the `path` program in a fresh machine allowing any interactive-only calls.
 ///
-/// `dir` has the same meaning as the parameter passed to `run_repl_loop`.
-fn run_interactive<P: AsRef<Path>>(path: P, dir: &Path) -> endbasic_core::exec::Result<i32> {
-    let mut machine = endbasic_std::MachineBuilder::default()
-        .make_interactive()
-        .with_store(new_store_with_demos(dir))
-        .build()?;
+/// `local_drive` is the optional local drive to mount and use as the default location.
+async fn run_interactive<P: AsRef<Path>>(
+    path: P,
+    local_drive_spec: &str,
+) -> endbasic_core::exec::Result<i32> {
+    let mut builder = endbasic_std::MachineBuilder::default().make_interactive();
+
+    let storage = builder.get_storage();
+    setup_storage(&mut storage.borrow_mut(), local_drive_spec)?;
+
+    let mut machine = builder.build()?;
     let mut input = File::open(path)?;
-    Ok(block_on(machine.exec(&mut input))?.as_exit_code())
+    Ok(machine.exec(&mut input).await?.as_exit_code())
 }
 
 /// Version of `main` that returns errors to the caller for reporting.
-fn safe_main(name: &str, args: env::Args) -> Result<i32> {
+async fn safe_main(name: &str, args: env::Args) -> Result<i32> {
     let args: Vec<String> = args.collect();
 
     let mut opts = Options::new();
     opts.optflag("h", "help", "show command-line usage information and exit");
     opts.optflag("i", "interactive", "force interactive mode when running a script");
-    opts.optopt("", "programs-dir", "directory where user programs are stored", "PATH");
+    opts.optopt("", "local-drive", "location of the drive to mount as LOCAL", "URI");
     opts.optflag("", "version", "show version information and exit");
     let matches = opts.parse(args)?;
 
@@ -171,24 +184,25 @@ fn safe_main(name: &str, args: env::Args) -> Result<i32> {
 
     match matches.free.as_slice() {
         [] => {
-            let programs_dir = get_programs_dir(matches.opt_str("programs-dir"))?;
-            Ok(run_repl_loop(&programs_dir)?)
+            let local_drive = get_local_drive_spec(matches.opt_str("local-drive"))?;
+            Ok(run_repl_loop(&local_drive).await?)
         }
         [file] => {
             if matches.opt_present("interactive") {
-                let programs_dir = get_programs_dir(matches.opt_str("programs-dir"))?;
-                Ok(run_interactive(file, &programs_dir)?)
+                let local_drive = get_local_drive_spec(matches.opt_str("local-drive"))?;
+                Ok(run_interactive(file, &local_drive).await?)
             } else {
-                Ok(run_script(file)?)
+                Ok(run_script(file).await?)
             }
         }
         [_, ..] => Err(UsageError::new("Too many arguments").into()),
     }
 }
 
-fn main() {
+#[tokio::main]
+async fn main() {
     let (name, args) = program_name(env::args(), "endbasic");
-    let exit_code = match safe_main(&name, args) {
+    let exit_code = match safe_main(&name, args).await {
         Ok(code) => code,
         Err(e) => {
             if let Some(e) = e.downcast_ref::<UsageError>() {
